/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright (c) 2012 Oracle and/or its affiliates. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License.  You can
 * obtain a copy of the License at
 * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
 * or packager/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at packager/legal/LICENSE.txt.
 *
 * GPL Classpath Exception:
 * Oracle designates this particular file as subject to the "Classpath"
 * exception as provided by Oracle in the GPL Version 2 section of the License
 * file that accompanied this code.
 *
 * Modifications:
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 *
 * Contributor(s):
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

package org.glassfish.osgicdi.impl;

import org.glassfish.osgijavaeebase.Extender;
import org.jboss.weld.bootstrap.WeldBootstrap;
import org.jboss.weld.bootstrap.api.Environments;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.BundleEvent;
import org.osgi.util.tracker.BundleTracker;
import org.osgi.util.tracker.BundleTrackerCustomizer;

import java.net.URL;
import java.util.Dictionary;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.logging.Logger;

/**
 * An extender that registers a BundleTracker to trigger Weld SPI directly
 * Pl.see http://java.net/jira/browse/GLASSFISH-19215
 *
 * @author Tang Yong(tangyong@cn.fujitsu.com)
 */
public class OSGiCDIExtender implements Extender {
    private static final Logger logger =
            Logger.getLogger(OSGiCDIExtender.class.getPackage().getName());
    private BundleContext context;
    //private OSGiCDIDeployer deployer;
    private BundleTracker tracker;
    private ExecutorService executorService;

    public OSGiCDIExtender(BundleContext context) {
        this.context = context;
    }

    public synchronized void start() {
        //deployer = new OSGiCDIDeployer(context);
        //deployer.register();
    	executorService = Executors.newSingleThreadExecutor();
    	tracker = new BundleTracker(context, Bundle.ACTIVE | Bundle.STARTING, new BeanBundleTrackerCustomizer());
        tracker.open();
    }

    public synchronized void stop() {
    	/*
        if (deployer != null) {
            deployer.unregister();
            deployer = null;
        }
        */
    	
    	if (tracker != null) tracker.close();
        tracker = null;
        executorService.shutdownNow();
    }
    
    private class BeanBundleTrackerCustomizer implements BundleTrackerCustomizer {

		@Override
		public Object addingBundle(final Bundle bundle, BundleEvent event) {
			final int state = bundle.getState();
			
            if (isReady(event, state) && (isCDIBundle(bundle))) {
                executorService.submit(new Runnable() {
					@Override
					public void run() {
						WeldBootstrap bootstrap = new WeldBootstrap();
				        OSGiBeanDeploymentArchiveFactory factory = new OSGiBeanDeploymentArchiveFactory();
				        OSGiBeanDeployment deployment = new OSGiBeanDeployment(bundle, bootstrap, factory);
				        bootstrap.startContainer(Environments.SE, deployment);
				        // Start the container
				        bootstrap.startInitialization();
				        bootstrap.deployBeans();
				        bootstrap.validateBeans();
				        bootstrap.endInitialization();						
					}
                });
                
                return bundle;
            }
            return null;
		}

		private boolean isCDIBundle(Bundle bundle) {
			final Dictionary<?, ?> headers = bundle.getHeaders();
	        URL url = bundle.getEntry(Constants.BEANS_XML_PATH);
	        return headers.get(Constants.EXPORT_EJB) == null && 
	        		headers.get(Constants.WEB_CONTEXT_PATH) == null && url != null &&
	                  headers.get(org.osgi.framework.Constants.FRAGMENT_HOST) == null;
		}

		private boolean isReady(BundleEvent event, int state) {
			return state == Bundle.ACTIVE ||
                    (state == Bundle.STARTING && (event != null && event.getType() == BundleEvent.LAZY_ACTIVATION));
		}

		@Override
		public void modifiedBundle(Bundle bundle, BundleEvent event,
				Object object) {
		}

		@Override
		public void removedBundle(Bundle bundle, BundleEvent event,
				Object object) {	
		}	
    }
}
